<?php
namespace Ep\Middleware;

use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\Response\HtmlResponse;
use Zend\Stratigility\Middleware\NotFoundHandler;
use Zend\Stratigility\MiddlewarePipe;
use Zend\Stratigility\MiddlewarePipeInterface;

/**
 * EP Aplikacija
 *
 * Ovo je u biti običan MiddlewarePipe koji dolazi već s nekakvim
 * middlewareovima.
 *
 * @package Ep\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class App implements MiddlewarePipeInterface
{
    /** @var ContainerInterface */
    private $container;
    /** @var array */
    private $config;
    /** @var MiddlewarePipe */
    private $pipeline;
    /** @var bool */
    private $composed = false;

    /**
     * Application constructor.
     *
     * @param ContainerInterface $container
     * @param array $config Konfiguracija aplikacije.
     *
     */
    public function __construct(ContainerInterface $container, array $config = [])
    {
        $this->pipeline = new MiddlewarePipe();

        $this->container = $container;
        $this->config = $config;
    }

    /**
     * @param ServerRequestInterface $request
     *
     * @return ResponseInterface
     */
    public function __invoke(ServerRequestInterface $request): ResponseInterface
    {
        return $this->handle($request);
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        if (!$this->composed) {
            $this->compose();
        }

        $this->pipeline->pipe(new NotFoundHandler(function() {
            return new HtmlResponse('404 Not found', 404);
        }));

        return $this->pipeline->process($request, $handler);
    }

    /**
     * @param MiddlewareInterface $middleware
     */
    public function pipe(MiddlewareInterface $middleware): void
    {
        if (!$this->composed) {
            $this->compose();
        }

        $this->pipeline->pipe($middleware);
    }

    /**
     * Handle the request and return a response.
     *
     * @param ServerRequestInterface $request
     *
     * @return ResponseInterface
     */
    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        if (!$this->composed) {
            $this->compose();
        }

        $this->pipeline->pipe(new NotFoundHandler(function() {
            return new HtmlResponse('404 Not found', 404);
        }));

        return $this->pipeline->handle($request);
    }

    /**
     * Slaže aplikaciju.
     */
    private function compose(): void
    {
        if ($this->config['error_handler'] ?? true) {
            /** @noinspection PhpUnhandledExceptionInspection */
            $this->pipeline->pipe($this->container->get('app.error_handler'));
        }

        if ($this->config['force_https'] ?? false) {
            /** @noinspection PhpUnhandledExceptionInspection */
            $this->pipeline->pipe($this->container->get('app.force_https'));
        }

        if ($this->config['force_www'] ?? false) {
            /** @noinspection PhpUnhandledExceptionInspection */
            $this->pipeline->pipe($this->container->get('app.force_www'));
        }

        /** @noinspection PhpUnhandledExceptionInspection */
        $this->pipeline->pipe($this->container->get('app.base_url'));

        $this->composed = true;
    }
}